basic_arithmetic = {
    "+": lambda arg: arg[1] + arg[0],
    "-": lambda arg: arg[1] - arg[0],
    "*": lambda arg: arg[1] * arg[0],
    "/": lambda arg: arg[1] / arg[0],
    "%": lambda arg: arg[1] % arg[0],
    "^": lambda arg: arg[1]**arg[0]
}